import { Input, Output, EventEmitter } from '@angular/core';
import { FormControl } from '@angular/forms';
import { FriendlyValidationErrorsService } from './../../services/friendly-validation-errors.service';

export abstract class NativeInputComponent {

	@Input()
	control: FormControl;

	@Input()
	set meta(meta: StringMap<any>) {
		this._meta = meta;
		this.exposeForTemplate();
	}

	@Output()
	call: EventEmitter<string> = new EventEmitter<string>();

	readonly componentName: string;
	exposeMetaInTemplate: string[] = [];
	_meta: StringMap<any>;

	constructor(protected valErrsService: FriendlyValidationErrorsService) {}

	exposeForTemplate() {
		// Move meta variables up a level, for direct access in templates
		this.exposeMetaInTemplate.map(p => this[p] = this._meta[p] !== undefined ? this._meta[p] : this[p]);
	}

	handle(fnId: string, val: any): void {
		this.call.emit(fnId); // Add value
	}

	handleChange(): void {
		if (this._meta.change) {
			this.handle(this._meta.change, this.control.value);
		}
	}

	getName(): string {
		return this.componentName;
	}

	getFirstFailureMsg(): string {
		if (!this.control.errors) {
			return '';
		}
		const key = Object.keys(this.control.errors)[0];
		return this._meta.valFailureMsgs[key] || this.valErrsService.getFriendly(key, this.control.errors[key]);
	}

}
